"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@yarnpkg/core");
const core_2 = require("@yarnpkg/core");
const plugin_essentials_1 = require("@yarnpkg/plugin-essentials");
const typescriptUtils_1 = require("./typescriptUtils");
const getTypesName = (descriptor) => {
    return descriptor.scope
        ? `${descriptor.scope}__${descriptor.name}`
        : `${descriptor.name}`;
};
const afterWorkspaceDependencyAddition = async (workspace, dependencyTarget, descriptor) => {
    if (descriptor.scope === `types`)
        return;
    const project = workspace.project;
    const configuration = project.configuration;
    const requiresInstallTypes = await typescriptUtils_1.hasDefinitelyTyped(descriptor, configuration);
    if (!requiresInstallTypes)
        return;
    const cache = await core_1.Cache.find(configuration);
    const typesName = getTypesName(descriptor);
    const target = plugin_essentials_1.suggestUtils.Target.DEVELOPMENT;
    const modifier = plugin_essentials_1.suggestUtils.Modifier.EXACT;
    const strategies = [plugin_essentials_1.suggestUtils.Strategy.LATEST];
    const request = core_2.structUtils.makeDescriptor(core_2.structUtils.makeIdent(`types`, typesName), `unknown`);
    const suggestions = await plugin_essentials_1.suggestUtils.getSuggestedDescriptors(request, { workspace, project, cache, target, modifier, strategies });
    const nonNullSuggestions = suggestions.filter(suggestion => suggestion.descriptor !== null);
    if (nonNullSuggestions.length === 0)
        return;
    const selected = nonNullSuggestions[0].descriptor;
    if (selected === null)
        return;
    workspace.manifest[target].set(selected.identHash, selected);
};
const afterWorkspaceDependencyRemoval = async (workspace, dependencyTarget, descriptor) => {
    if (descriptor.scope === `types`)
        return;
    const target = plugin_essentials_1.suggestUtils.Target.DEVELOPMENT;
    const typesName = getTypesName(descriptor);
    const ident = core_2.structUtils.makeIdent(`types`, typesName);
    const current = workspace.manifest[target].get(ident.identHash);
    if (typeof current === `undefined`)
        return;
    workspace.manifest[target].delete(ident.identHash);
};
const beforeWorkspacePacking = (workspace, rawManifest) => {
    if (rawManifest.publishConfig && rawManifest.publishConfig.typings)
        rawManifest.typings = rawManifest.publishConfig.typings;
    if (rawManifest.publishConfig && rawManifest.publishConfig.types) {
        rawManifest.types = rawManifest.publishConfig.types;
    }
};
const plugin = {
    hooks: {
        afterWorkspaceDependencyAddition,
        afterWorkspaceDependencyRemoval,
        beforeWorkspacePacking,
    },
};
// eslint-disable-next-line arca/no-default-export
exports.default = plugin;
